operation CRk(qctrl: Qubit, k: Int, qtarget: Qubit) : Unit is Adj {
    body(...) {
        let theta = 2.0 * Math.PI() / Convert.IntAsDouble(2 ^ k);
        Controlled R1([qctrl], (theta, qtarget));
    }
    adjoint auto;
}

operation Reverse(qs : Qubit[]) : Unit is Adj {
    body(...) {
        let n = Length(qs);
        for i in 0 .. n / 2 - 1
        { SWAP(qs[i], qs[n - i - 1]); }
    }
    adjoint auto;
}

operation QFT(qs : Qubit[]) : Unit is Adj {
    body(...) {
        let n = Length(qs);
        for i in 0 .. n - 2 {
            H(qs[i]);
            for j in i + 1 .. n - 1 {
                let k = j - i + 1;
                CRk(qs[j], k, qs[i]);   // Call "CRk"
            }
        }
        H(qs[n - 1]);
        Reverse(qs);    // Call "Reverse"
    }
    adjoint auto;
}